/***
 * Parser for .log and .oxy files.
 * ---------------------------------
 * author: Ryan Schmitt
 * date: 2-19-11
 */

#include "SciDataParser.h"


int
SciDataParser::parseFile(std::string const &data) {

   // read the files into buffers
   char* dataBuf = textFileRead(data.c_str());

   float x, y, z, tr;
   char * tok;
   int line = 0;
   tok = strtok(dataBuf, "\n");

   // instantiate beginning min/max values
   m_minLoc = Vector3(DBL_MAX,DBL_MAX,DBL_MAX);
   m_maxLoc = Vector3(-DBL_MAX,-DBL_MAX,-DBL_MAX);
   m_minTR = DBL_MAX;
   m_maxTR = -DBL_MAX;

   z = -1.0;

   while(tok != NULL) {
	  char *cA = (char*)malloc(sizeof(char));
	  char *cl = (char*)malloc(sizeof(char));
	  char *me = (char*)malloc(sizeof(char));
	  char *car = (char*)malloc(sizeof(char));
      // scan in line information.  panic if the data does not match
      if(sscanf(tok, "%f %f %f %s %s %s %s", &y, &x, &tr, cA, cl, me, car) <= 0) {
         printf("Error: Malformed file on line %d\n", line);
         exit(1);
      }

      // update minimum location
      if(x < m_minLoc.x) m_minLoc.x = x;
      if(y < m_minLoc.y) m_minLoc.y = y;
      if(z < m_minLoc.z) m_minLoc.z = z;

      // update maximum location
      if(x > m_maxLoc.x) m_maxLoc.x = x;
      if(y > m_maxLoc.y) m_maxLoc.y = y;
      if(z > m_maxLoc.z) m_maxLoc.z = z;

      // update min/max totalReleases values
      if(tr < m_minTR) m_minTR = tr;
      if(tr > m_maxTR) m_maxTR = tr;

      // push scidata into list
      m_data.push_back(SciData(x, y, z, tr, cA, cl, me, car));
	  s_data.push_back(SciData(x, y, z, tr, cA, cl, me, car));

      // move onto next line
      tok = strtok(NULL, "\n");
      line++;
   }

    // return how many data items were read in
    return m_data.size();
}
