/* Final Project for CSC 572
   Author: John Fulton

   Description of Operation:
   1) Read Data from Elevation File - no data yet
   2) Read Data from Toxins File
   3) Display California Terrain
   4) Overlay Toxins Data
   5) Use keys to control display
		a - highlight clean air chemicals
		c - highlight carcinogen chemicals
		m - highlight metals
		p - highlight PBT chemicals
		r - reset display
		q - quit
		x - zoom out
		z - zoom in
		Arrow Keys - shift about the map
*/
#include "final_project.h"

void keyOperations (void) {
	for(int i = 0; i < 255; i++) {		//for each possible key action
		if( keyStates[i] == true ) {	//determine if it is pressed
			printf("%d\n", i);
			switch(i) {					//carry out the appropriate functionality
				case 'a': case 'A':
					cA_flag = true;
					PBT_flag = false;
					metal_flag = false;
					carn_flag = false;
					break;
				case 'c': case 'C':
					cA_flag = false;
					PBT_flag = false;
					metal_flag = false;
					carn_flag = true;
					break;
				case 'm': case 'M':
					cA_flag = false;
					PBT_flag = false;
					metal_flag = true;
					carn_flag = false;
					break;
				case 'p': case 'P':
					cA_flag = false;
					PBT_flag = true;
					metal_flag = false;
					carn_flag = false;
					break;
				case 'q': case 'Q':
					exit(0);
				case 'r': case 'R':
					shiftx = 0.0;
					shifty = 0.0;
					zoom = 1.0;
					cA_flag = false;
					PBT_flag = false;
					metal_flag = false;
					carn_flag = false;
					break;
				case 'x': case 'X':
					zoom = std::min(1.0,zoom+0.05);
					break;
				case 'z': case 'Z':
					zoom = std::max(0.25,zoom-0.05);
					break;
				default : ;
			}//end switch
			keyStates[i] = false;
		}//end if
	}//end for
}
void keySpecialOperations (void) {
	for(int i = 0; i < 255; i++) {		//for each possible key action
		if( keySpecialStates[i] == true ) {	//determine if it is pressed
			printf("%d\n", i);
			switch(i) {					//carry out the appropriate functionality
				case GLUT_KEY_UP:
					shifty += 0.01;
					break;
				case GLUT_KEY_DOWN:
					shifty -= 0.01;
					break;
			    case GLUT_KEY_LEFT:
					shiftx -= 0.01;
					break;
			    case GLUT_KEY_RIGHT:
					shiftx += 0.01;
					break;
				default : ;
			}//end switch
		}//end if
	}//end for
}
void drawDataPoint (float x, float y, float z) {
	glBegin(GL_POINTS);				
	glVertex3f(x,y,z);
	glEnd();
}
void drawOverlay(void) {
for(int i = 0; i < data.size(); i++) {
		Vector3 tmp = data[i].getLocation();
	
		tmp.x = ((tmp.x - (-126.1)) / (-113.4+126.1) * 4.0) - 2.0;
		tmp.y = ((tmp.y - (31.0)) / (42.67-31.0) * 4.0) - 2.0;
		tmp.z = 0;
		
		float size = std::max((10/zoom*(data[i].getTotalRelease() - minTR) / maxTR),2.0);

		glPointSize(size);

		if      (cA_flag	 && strcmp(data[i].getCleanAir(), "YES") == 0)			{ 
			glColor3f(1,0,0); // red
			drawDataPoint(tmp.x,tmp.y,tmp.z);
		}
		else if (PBT_flag	 && strcmp(data[i].getClassification(), "PBT") == 0)	{ 
			glColor3f(1,0,0); // green
			drawDataPoint(tmp.x,tmp.y,tmp.z);
		}
		else if (metal_flag  && strcmp(data[i].getMetal(), "YES") == 0)				{ 
			glColor3f(1,0,0); // blue
			drawDataPoint(tmp.x,tmp.y,tmp.z);
		}
		else if (carn_flag   && strcmp(data[i].getCarcinogen(), "YES") == 0)		{ 
			glColor3f(1,0,0); // purple
			drawDataPoint(tmp.x,tmp.y,tmp.z);
		}
		else if (!cA_flag && !PBT_flag && !metal_flag && !carn_flag) { 
			glColor3f(1,1,1); //white
			drawDataPoint(tmp.x,tmp.y,tmp.z);
		}
	}
	glColor3f(1,1,1); //white
}
GLuint loadTexture( const char * filename, int width, int height )
{
	GLuint texture;
	unsigned char* data;
	FILE* file;
	
	file = fopen(filename, "rb");
	if(file == NULL) return 0;
	data = (unsigned char*) malloc (width*height*3);
	fread(data, width*height*3, 1, file);
	fclose(file);

    glGenTextures(1, &texture);
	glBindTexture(GL_TEXTURE_2D, texture);
	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);

    glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR );
    glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );
    //Here we are setting the parameter to repeat the texture instead of clamping the texture
    //to the edge of our shape. 
    glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT );
    glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT );
    //Generate the texture
    glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, width, height, 0, GL_RGB, GL_UNSIGNED_BYTE, data);
    free( data ); //free the texture
    return texture; //return whether it was successfull
}
void FreeTexture( GLuint texture )
{
  glDeleteTextures( 1, &texture ); 
}
void square (void) {
	glEnable( GL_TEXTURE_2D );
	glBindTexture( GL_TEXTURE_2D, texture ); //bind our texture to our shape
    
	glBegin (GL_QUADS);
	glTexCoord2d(0.0,0.0); glVertex2d(-2.0,2.0);
	glTexCoord2d(1.0,0.0); glVertex2d(2.0,2.0);
	glTexCoord2d(1.0,1.0); glVertex2d(2.0,-2.0);
	glTexCoord2d(0.0,1.0); glVertex2d(-2.0,-2.0);
    glEnd();

	glDisable(GL_TEXTURE_2D);
}
void display (void) {
	keyOperations();								//take care of key actions
	keySpecialOperations();							//take care of special key actions;
	glClear(GL_COLOR_BUFFER_BIT);					//clear the color buffer
	glLoadIdentity();								//load the identity matrix to reset drawing location
	gluLookAt(shiftx,shifty,4.0f*zoom,
			  shiftx,shifty,0.0f,
			  0.0f,1.0f,0.0f);

	glPushMatrix();
	
	int width = 700;
	int height = 827;
	texture = loadTexture("california.raw",width,height);
	square();
	
	drawOverlay();

    glPopMatrix();

	FreeTexture(texture);

	glutSwapBuffers();								//Exchange the two buffers
}
void reshape (int width, int height) {
	glViewport(0, 0, (GLsizei)width, (GLsizei)height);					//Set up our viewport
	glMatrixMode(GL_PROJECTION);										//Switch to the projection matrix
	glLoadIdentity();													//Reset the Projection Matrix
	gluPerspective(60, (GLfloat)width / (GLfloat)height, 1.0, 100.0);	//Set the Camera
	glMatrixMode(GL_MODELVIEW);											//Switch to the model view matrix
}
void keyPressed (unsigned char key, int x, int y) {
	keyStates[key] = true; //set the state that the key is pressed
	switch(key) {
		default : ;
	}
}
void keyUp (unsigned char key, int x, int y) {
	keyStates[key] = false; //Set the state that the key is not pressed
	switch(key) {
		case 'n': case 'N':
			num = (num + 1)%data.size();
			break;
		default : ;
	}
}
void keySpecial (int key, int x, int y) {
	keySpecialStates[key] = true; //set the state that the key is pressed
	switch(key) {
		default : ;
	}
}
void keySpecialUp (int key, int x, int y) {
	keySpecialStates[key] = false; //Set the state that the key is not pressed
	switch(key) {
		default : ;
	}
}
void initializeDisplay(void){
	glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGBA);	//Set up two basic display buffers
	glutInitWindowSize(500,500);					//Set the width and height of the window
	glutInitWindowPosition(0,0);				//Set the position of the window
	glutCreateWindow("Visualization of 2007 Toxics Release Inventory Data for the State of California"); //Set the title for the window
	glutDisplayFunc(display);						//call our display function
	glutIdleFunc(display);							//let display function also serve as idle function
	glutReshapeFunc(reshape);						//call our reshape function
	glutKeyboardFunc(keyPressed);					//call our key pressed function
	glutKeyboardUpFunc(keyUp);						//call our key released function
	glutSpecialFunc(keySpecial);					//call our special key pressed function
	glutSpecialUpFunc(keySpecialUp);				//call our special key released function

	zoom = 1.0;
}
void aquireData(void) {
	SciDataParser parser;
	parser.parseFile("data/Data_mod2.txt");
	data = parser.getData();
	Vector3 tmp;

	for(int i = 0; i < data.size(); i++) {
		tmp = data[i].getLocation();
		printf("<%f, %f, %f>\t%f\n", tmp.x, tmp.y, tmp.z, data[i].getTotalRelease());
	}

	minLoc = parser.getMinLoc();
	maxLoc = parser.getMaxLoc();
	center.x = minLoc.x + (maxLoc.x - minLoc.x)/2.0;
	center.y = minLoc.y + (maxLoc.y - minLoc.y)/2.0;
	center.z = minLoc.z + (maxLoc.z - minLoc.z)/2.0;
	max_extent = maxLoc.x - minLoc.x;

	minTR = parser.getMinTR();
	maxTR = parser.getMaxTR();

	printf("\n");
	printf("Min Location: <%f, %f, %f>\n", minLoc.x, minLoc.y, minLoc.z);
	printf("Max Location: <%f, %f, %f>\n", maxLoc.x, maxLoc.y, maxLoc.z);
	printf("Min TR Level: %f\n", minTR);
	printf("Max TR Level: %f\n", maxTR);
}
int main (int argc, char** argv) {
	glutInit(&argc, argv);							//Initialize Glut
	
	initializeDisplay();
	aquireData();

	glutMainLoop();
}