//super simple OGL/glut code paired with parser (C. Gibson, R. Schmitt, K. Blizard) 
//to read oxygen diffusion data for 572
#include <stdlib.h>
//for apple
#include <GL/glew.h>
#include <GL/glut.h>
#include <stdio.h>
#include <math.h>
#include <assert.h>
#include <vector>

#include "SciDataParser.h"
#include "SciData.h"
#include "SciDataParser.h"
#include "Vector3.h"

using std::string;
using std::vector;

int GW;
int GH;
//some useful data
Vector3 minLoc, maxLoc, center;
double minO2, maxO2;
vector<SciData> data; 
float max_extent;
float color_range;

void drawSphere() {
  glutSolidSphere(0.65, 10, 10);
}

void display() {
  
   Vector3 tmp;
  float intense;

  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
  
  glMatrixMode(GL_MODELVIEW);
  //save current matrix
  glPushMatrix();

  //set up the camera - watch these variables
  gluLookAt(5.5, 1.0, 1.0, //eye position
    0.0, 0.0, 0.0, //point we are looking at
    0.0, 1.0, 0.0);  //up vector

  //TODO insert code to draw the oxygen diffusion data as spheres and
  //reasonably colored. - watch out for scale and loaction of the data
  //relative to your world

  glPointSize(3.0f);
  glBegin(GL_POINTS);
   
  for(int i = 0; i < data.size(); i++) {
	Vector3 tmp = data[i].getLocation();
	
	tmp.x = ((tmp.x - minLoc.x) / max_extent * 4.0) - 1;
	tmp.y = ((tmp.y - minLoc.y) / max_extent * 4.0) - 1;
	tmp.z = ((tmp.z - minLoc.z) / max_extent * 4.0) - 1;
	
	float color = (data[i].getO2Concenration() - minO2) / color_range;

	 if      (color <= .2) glColor3f(0,0,1); // blue
     else if (color <= .4) glColor3f(0,1,0); // green
     else if (color <= .6) glColor3f(1,1,0); // red
     else if (color <= .8) glColor3f(1,0,1); // purple
     else                  glColor3f(1,0,0); // white

	glVertex3f(tmp.x, tmp.y, tmp.z);
    //glTranslatef(tmp.x,tmp.y,tmp.z);
	//drawSphere();
  }
  glEnd();

  glPopMatrix();
  
  glutSwapBuffers();
  
}

void reshape(int w, int h) {
  GW = w;
  GH = h;
  
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  //notice the change in the near and far planes - they are measure with respect to the
  //camera position
  glOrtho( -2*(float)w/h, (float)2*w/h, -2, 2, -1.0, 150.0);
  glMatrixMode(GL_MODELVIEW);
  glViewport(0, 0, w, h);
  
}


void keyboard(unsigned char key, int x, int y )
{
  switch( key ) {
  case 'q': case 'Q' :
    exit( EXIT_SUCCESS );
    break;
  }
  glutPostRedisplay();
}

int main(int argc, char** argv) {

   SciDataParser parser;

   parser.parseFile("data/HV_Grid_Undulating_Oxy_01.txt");

   data = parser.getData();

   Vector3 tmp;

   for(int i = 0; i < data.size(); i++) {

      tmp = data[i].getLocation();
      printf("<%f, %f, %f>\t%f\n", tmp.x, tmp.y, tmp.z, data[i].getO2Concenration());

   }
   
   minLoc = parser.getMinLoc();
   maxLoc = parser.getMaxLoc();
   center.x = minLoc.x + (maxLoc.x - minLoc.x)/2.0;
   center.y = minLoc.y + (maxLoc.y - minLoc.y)/2.0;
   center.z = minLoc.z + (maxLoc.z - minLoc.z)/2.0;
   max_extent = maxLoc.x - minLoc.x;

   minO2 = parser.getMinO2();
   maxO2 = parser.getMaxO2();
   color_range = maxO2-minO2;

   printf("\n");
   printf("Min Location: <%f, %f, %f>\n", minLoc.x, minLoc.y, minLoc.z);
   printf("Max Location: <%f, %f, %f>\n", maxLoc.x, maxLoc.y, maxLoc.z);
   printf("Min O2 Level: %f\n", minO2);
   printf("Max O2 Level: %f\n", maxO2);

  //set up glut window
  glutInit(&argc, argv);
  glutInitWindowSize(400, 400);
  glutInitDisplayMode( GLUT_RGB | GLUT_DOUBLE | GLUT_DEPTH);
  glutCreateWindow("Example OGL");
  glClearColor(0.0, 0.0, 0.0, 1.0);
  
  //set up glut callbacks
  glutDisplayFunc(display);
  glutReshapeFunc(reshape);
  glutKeyboardFunc(keyboard);
  
  //initialize globals
  GW = 400;
  GH = 400;
  
  //enable GL features we want
  glEnable(GL_DEPTH_TEST);
  
  glutMainLoop();
  
}


